@@ -44,6 +44,8 @@ class Child:
         "last_ping",
         "soft_timeout_reached",
         "done",
+        "soft_timeout",
+        "hard_timeout",
     ]
 
     def __init__(self, pid, stdin, pingin, broadcastout):
@@ -57,6 +59,8 @@ class Child:
         self.soft_timeout_reached = False
         self.last_ping = time.time()
         self.done = False
+        self.soft_timeout = None
+        self.hard_timeout = None
 
     def close(self):
         close_fd(self.pingin)
@@ -198,29 +202,49 @@ class Worker:
 
             self._num_tasks_processed = 0
             while not self._stop_processing:
-                queue_name, payload = self.wakaq._blocking_dequeue()
+                self._send_ping_to_parent()
+                try:
+                    queue_broker_key, payload = self.wakaq._blocking_dequeue()
+                except SoftTimeout:
+                    payload = None
                 if payload is not None:
-                    task = self.wakaq.tasks[payload["name"]]
-                    retry = payload.get("retry") or 0
                     try:
-                        self._execute_task(task, payload)
-                    except SoftTimeout:
-                        retry += 1
-                        max_retries = task.max_retries
-                        if max_retries is None:
-                            queue = self.wakaq.queues_by_name[queue_name]
-                            max_retries = (
-                                queue.default_max_retries
-                                if queue.default_max_retries is not None
-                                else self.wakaq.default_max_retries
-                            )
-                        if retry > max_retries:
-                            raise
-                        self.wakaq._enqueue_at_end(
-                            task.name, queue_name, payload["args"], payload["kwargs"], retry=retry
-                        )
+                        task = self.wakaq.tasks[payload["name"]]
+                    except KeyError:
+                        log.error(f'Task not found: {payload["name"]}')
+                        task = None
+
+                    if task is not None:
+                        current_task.set((task, payload))
+                        retry = payload.get("retry") or 0
+                        try:
+                            self._execute_task(task, payload)
+                        except SoftTimeout:
+                            retry += 1
+                            queue = self.wakaq.queues_by_key[queue_broker_key]
+                            max_retries = task.max_retries
+                            if max_retries is None:
+                                max_retries = (
+                                    queue.max_retries if queue.max_retries is not None else self.wakaq.max_retries
+                                )
+                            if retry > max_retries:
+                                log.error(traceback.format_exc())
+                            else:
+                                log.warning(traceback.format_exc())
+                                self.wakaq._enqueue_at_end(
+                                    task.name, queue.name, payload["args"], payload["kwargs"], retry=retry
+                                )
+                        except:
+                            log.error(traceback.format_exc())
+                        finally:
+                            current_task.set(None)
+
+                    else:
+                        self._send_ping_to_parent()
+
                 else:
                     self._send_ping_to_parent()
+                flush_fh(sys.stdout)
                 self._execute_broadcast_tasks()
                 if self.wakaq.max_tasks_per_worker and self._num_tasks_processed >= self.wakaq.max_tasks_per_worker:
                     log.info(f"restarting worker after {self._num_tasks_processed} tasks")
@@ -238,8 +262,8 @@ class Worker:
             close_fd(self._stdout)
             close_fd(self._pingout)
 
-    def _send_ping_to_parent(self):
-        write_fd_or_raise(self._pingout, ".")
+    def _send_ping_to_parent(self, task_name=None):
+        write_fd_or_raise(self._pingout, f"{task_name or ''}\n")
 
     def _add_child(self, pid, stdin, pingin, broadcastout):
         self.children.append(Child(pid, stdin, pingin, broadcastout))
@@ -289,8 +313,7 @@ class Worker:
                 self.wakaq._enqueue_at_front(task_name, queue.name, args, kwargs)
 
     def _execute_task(self, task, payload):
-        self._send_ping_to_parent()
-        current_task.set(task)
+        self._send_ping_to_parent(task_name=task.name)
         log.debug(f"running with payload {payload}")
         if self.wakaq.before_task_started_callback:
             self.wakaq.before_task_started_callback()
@@ -299,10 +322,7 @@ class Worker:
                 self.wakaq.wrap_tasks_function(task.fn)(*payload["args"], **payload["kwargs"])
             else:
                 task.fn(*payload["args"], **payload["kwargs"])
-        except:
-            log.error(traceback.format_exc())
         finally:
-            current_task.set(None)
             self._send_ping_to_parent()
             self._num_tasks_processed += 1
             if self.wakaq.after_task_finished_callback:
@@ -314,19 +334,33 @@ class Worker:
             return
         for payload in payloads.splitlines():
             payload = deserialize(payload)
-            task = self.wakaq.tasks[payload["name"]]
+            try:
+                task = self.wakaq.tasks[payload["name"]]
+            except KeyError:
+                log.error(f'Task not found: {payload["name"]}')
+                continue
             retry = 0
-            while True:
-                try:
-                    self._execute_task(task, payload)
-                    break
-                except SoftTimeout:
-                    retry += 1
-                    max_retries = task.max_retries
-                    if max_retries is None:
-                        max_retries = self.wakaq.default_max_retries
-                    if retry > max_retries:
-                        raise
+            current_task.set((task, payload))
+            try:
+                while True:
+                    try:
+                        self._execute_task(task, payload)
+                        break
+                    except SoftTimeout:
+                        retry += 1
+                        max_retries = task.max_retries
+                        if max_retries is None:
+                            max_retries = self.wakaq.max_retries
+                        if retry > max_retries:
+                            log.error(traceback.format_exc())
+                            break
+                        else:
+                            log.warning(traceback.format_exc())
+                    except:
+                        log.error(traceback.format_exc())
+                        break
+            finally:
+                current_task.set(None)
 
     def _read_child_logs(self):
         for child in self.children:
@@ -362,13 +396,26 @@ class Worker:
             if ping != "":
                 child.last_ping = time.time()
                 child.soft_timeout_reached = False
-            elif self.wakaq.soft_timeout or self.wakaq.hard_timeout:
-                runtime = time.time() - child.last_ping
-                if self.wakaq.hard_timeout and runtime > self.wakaq.hard_timeout:
-                    kill(child.pid, signal.SIGKILL)
-                elif not child.soft_timeout_reached and self.wakaq.soft_timeout and runtime > self.wakaq.soft_timeout:
-                    child.soft_timeout_reached = True  # prevent raising SoftTimeout twice for same child
-                    kill(child.pid, signal.SIGQUIT)
+                ping = ping.split("\n")
+                if len(ping) > 0 and ping[-1]:
+                    task_name, queue_name = ping[-1].split(":")
+                    task = self.wakaq.tasks[task_name]
+                    queue = self.wakaq.queues_by_name[queue_name]
+                    child.soft_timeout = task.soft_timeout or queue.soft_timeout or self.wakaq.soft_timeout
+                    child.hard_timeout = task.hard_timeout or queue.hard_timeout or self.wakaq.hard_timeout
+                else:
+                    child.soft_timeout = self.wakaq.soft_timeout
+                    child.hard_timeout = self.wakaq.hard_timeout
+            else:
+                soft_timeout = child.soft_timeout or self.wakaq.soft_timeout
+                hard_timeout = child.hard_timeout or self.wakaq.hard_timeout
+                if soft_timeout or hard_timeout:
+                    runtime = time.time() - child.last_ping
+                    if hard_timeout and runtime > hard_timeout:
+                        kill(child.pid, signal.SIGKILL)
+                    elif not child.soft_timeout_reached and soft_timeout and runtime > soft_timeout:
+                        child.soft_timeout_reached = True  # prevent raising SoftTimeout twice for same child
+                        kill(child.pid, signal.SIGQUIT)
 
     def _listen_for_broadcast_task(self):
         msg = self._pubsub.get_message(ignore_subscribe_messages=True, timeout=self.wakaq.wait_timeout)
